@@ -21,6 +21,8 @@ includes:
 
  * Scrolling the screen up or down
 
+ * Clickable hyperlinks to URIs
+
  * Changing the title of the terminal
 
 A terminal that supports control character sequences acts on them when they
@@ -119,7 +121,7 @@ Another example:
 >   setSGR [Reset]  -- reset to default colour scheme
 
 For many more examples, see the project's extensive
-<https://github.com/feuerbach/ansi-terminal/blob/master/app/Example.hs Example.hs> file.
+<https://github.com/UnkindPartition/ansi-terminal/blob/master/app/Example.hs Example.hs> file.
 -}
 #if defined(WINDOWS)
 module System.Console.ANSI
